#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>
#include "Adafruit_BME280.h"
#include <Wire.h>
#include <Adafruit_SSD1306.h>
Adafruit_BME280 bme;




//Sets the name for the sensor.
//Tip: Keep the name short, otherwise it won't fit on the screen
char sensorName[] = "Remote";

//Choose whether you want the temperature in celsius or not.
boolean useCelsius = false;




//OLED screen things
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define OLED_RESET -1
#define SCREEN_ADDRESS 0x3C
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

//NRF24 things
RF24 radio(14, 16);
const byte address[6] = "00001";

struct package {
  float temperature;
  float humidity;
};
typedef struct package Package;
Package data;

float remoteTemp;
float remoteHumid;

float localTemp;
float localHumid;

int currentSensor = 0;
unsigned long lastSystemTime = millis();

int leftButton = 27;
int menuButton = 26;
int rightButton = 25;
long LBLastPress = 0;
long RBLastPress = 0;

void IRAM_ATTR cycleMenusInterrupt() {
  if (digitalRead(leftButton) == LOW && millis() > LBLastPress + 200) {
    LBLastPress = millis();
    currentSensor--;
    if (currentSensor < 0) {
      currentSensor = 1;
    }
  }
  if (digitalRead(rightButton) == LOW && millis() > RBLastPress + 200) {
    RBLastPress = millis();
    currentSensor++;
    if (currentSensor > 1) {
      currentSensor = 0;
    }
  }
}

void setup() {
  //NRF24 Setup
  radio.begin();
  radio.setPALevel(RF24_PA_MAX);
  radio.openReadingPipe(1, address);
  radio.startListening();
  //OLED display initialization stuff
  if (!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 allocation failed"));
    for (;;)
      ;
  }
  display.clearDisplay();
  delay(250);
  display.display();
  delay(250);
  display.clearDisplay();
  //Buttons
  pinMode(leftButton, INPUT_PULLUP);
  pinMode(menuButton, INPUT_PULLUP);
  pinMode(rightButton, INPUT_PULLUP);
  attachInterrupt(leftButton, cycleMenusInterrupt, FALLING);
  attachInterrupt(rightButton, cycleMenusInterrupt, FALLING);
  //Sensor
  unsigned status;
  status = bme.begin(0x76);
}

void loop() {
  //Listens for incoming data from the remote sensor. Also refreshes local sensor data
  if (millis() >= lastSystemTime + 5000) {
    getRemoteData();
    getLocalData();
    lastSystemTime = millis();
  }
  sensorDisplay(currentSensor);
}

void getLocalData() {  //Gathers data from local sensor
  localTemp = bme.readTemperature();
  localHumid = bme.readHumidity();
}

void getRemoteData() {
  if (radio.available()) {
    while (radio.available()) {
      radio.read(&data, sizeof(data));
      radio.flush_rx();
      remoteTemp = data.temperature;
      remoteHumid = data.humidity;
    }
  }
}


void sensorDisplay(int sensorNumber) {  //Displays the main screen for any given sensor
  float tempData;
  if (sensorNumber == 0) {
    if (useCelsius == true) {
      tempData = localTemp;
    } else {
      tempData = (localTemp * 1.8) + 32;
    }
    display.clearDisplay();
    display.setTextColor(SSD1306_WHITE);
    display.setTextSize(2);
    display.setCursor(0, 0);
    display.println("Local Sens");
    display.print("T: ");
    display.print(tempData, 2);
    if (useCelsius == true) {
      display.println("C");
    } else {
      display.println("F");
    }
    display.print("H: ");
    display.print(localHumid, 2);
    display.println("%");
    display.display();
    return;
  }
  if (useCelsius == true) {
    tempData = remoteTemp;
  } else {
    tempData = (remoteTemp * 1.8) + 32;
  }
  display.clearDisplay();
  display.setTextColor(SSD1306_WHITE);
  display.setTextSize(2);
  display.setCursor(0, 0);
  display.println(sensorName);
  display.print("T: ");
  display.print(tempData, 2);
  if (useCelsius == true) {
    display.println("C");
  } else {
    display.println("F");
  }
  display.print("H: ");
  display.print(remoteHumid, 2);
  display.println("%");
  display.display();
}